#!/usr/bin/ruby
#  encoding: UTF-8
#
#   metrics-memory
#
# DESCRIPTION:
#
# OUTPUT:
#   metric data
#
# PLATFORMS:
#   Linux
#
# DEPENDENCIES:
#   gem: sensu-plugin
#
# USAGE:
#   ./check-mssql-lag.rb
#

require 'sensu-plugin/check/cli'
require 'tiny_tds'

class CheckMsSQLLag < Sensu::Plugin::Check::CLI
  option(:primary_host,
         short: '-p',
         long: '--primary-host=HOST',
         description: 'Primary HOST')

  option(:replica_host,
         short: '-r',
         long: '--replica_host=HOST',
         description: 'Replica HOST',
         default: 'localhost')

  option(:port,
         short: '-P',
         long: '--port=PORT',
         description: 'MsSQL port',
         default: 1433)

  option(:database,
         short: '-d',
         long: '--database=NAME',
         description: 'Database NAME')

  option(:user,
         short: '-u',
         long: '--user=USER',
         description: 'Database USER')

  option(:password,
         short: '-P',
         long: '--password=PASSWORD',
         description: 'Database PASSWORD')

  option(:ssl,
         short: '-S',
         long: '--ssl',
         boolean: true,
         description: 'Require SSL')

  option(:warning,
         short: '-w',
         long: '--warning=VALUE',
         description: 'Warning threshold for replication lag (in MB)',
         proc: proc(&:to_i),
         default: 60)

  option(:critical,
         short: '-c',
         long: '--critical=VALUE',
         description: 'Critical threshold for replication lag (in seconds)',
         proc: proc(&:to_i),
         default: 300)

  option(:timeout,
         short: '-T',
         long: '--timeout',
         default: nil,
         description: 'Connection timeout (seconds)')

  def run
    delay = calculate_delay
    critical if delay >= config[:critical]
    warning if delay >= config[:warning]
    ok
  end

  def calculate_delay
    primary = TinyTds::Client.new(username: config[:user], password: config[:password],
                                  host: config[:primary_host], database: config[:database],
                                  port: config[:port])
    replica = TinyTds::Client.new(username: config[:user], password: config[:password],
                                  host: config[:replica_host], database: config[:database],
                                  port: config[:port])
    sql = "SELECT TOP 1 DATEDIFF(SECOND, '1970-01-01', CreatedAt) As lag FROM [dbo].[UserGeoLocationHistories] ORDER BY CreatedAt DESC;"
    primary.execute(sql).each[0]['lag'] - replica.execute(sql).each[0]['lag']
  end

end
